/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.refactoring.rolemapping.resource.rolemapping.mopp;

public class RolemappingPrinter implements org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTextPrinter {
	
	protected org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolverFactory tokenResolverFactory = new org.emftext.language.refactoring.rolemapping.resource.rolemapping.mopp.RolemappingTokenResolverFactory();
	
	protected java.io.OutputStream outputStream;
	
	/**
	 * Holds the resource that is associated with this printer. This may be null if
	 * the printer is used stand alone.
	 */
	private org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTextResource resource;
	
	private java.util.Map<?, ?> options;
	private String encoding = System.getProperty("file.encoding");
	
	public RolemappingPrinter(java.io.OutputStream outputStream, org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTextResource resource) {
		super();
		this.outputStream = outputStream;
		this.resource = resource;
	}
	
	protected int matchCount(java.util.Map<String, Integer> featureCounter, java.util.Collection<String> needed) {
		int pos = 0;
		int neg = 0;
		
		for (String featureName : featureCounter.keySet()) {
			if (needed.contains(featureName)) {
				int value = featureCounter.get(featureName);
				if (value == 0) {
					neg += 1;
				} else {
					pos += 1;
				}
			}
		}
		return neg > 0 ? -neg : pos;
	}
	
	protected void doPrint(org.eclipse.emf.ecore.EObject element, java.io.PrintWriter out, String globaltab) {
		if (element == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write.");
		}
		if (out == null) {
			throw new java.lang.IllegalArgumentException("Nothing to write on.");
		}
		
		if (element instanceof org.emftext.language.refactoring.rolemapping.RoleMappingModel) {
			print_org_emftext_language_refactoring_rolemapping_RoleMappingModel((org.emftext.language.refactoring.rolemapping.RoleMappingModel) element, globaltab, out);
			return;
		}
		if (element instanceof org.emftext.language.refactoring.rolemapping.RoleMapping) {
			print_org_emftext_language_refactoring_rolemapping_RoleMapping((org.emftext.language.refactoring.rolemapping.RoleMapping) element, globaltab, out);
			return;
		}
		if (element instanceof org.emftext.language.refactoring.rolemapping.ConcreteMapping) {
			print_org_emftext_language_refactoring_rolemapping_ConcreteMapping((org.emftext.language.refactoring.rolemapping.ConcreteMapping) element, globaltab, out);
			return;
		}
		if (element instanceof org.emftext.language.refactoring.rolemapping.CollaborationMapping) {
			print_org_emftext_language_refactoring_rolemapping_CollaborationMapping((org.emftext.language.refactoring.rolemapping.CollaborationMapping) element, globaltab, out);
			return;
		}
		if (element instanceof org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair) {
			print_org_emftext_language_refactoring_rolemapping_ReferenceMetaClassPair((org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair) element, globaltab, out);
			return;
		}
		if (element instanceof org.emftext.language.refactoring.rolemapping.AttributeMapping) {
			print_org_emftext_language_refactoring_rolemapping_AttributeMapping((org.emftext.language.refactoring.rolemapping.AttributeMapping) element, globaltab, out);
			return;
		}
		
		addWarningToResource("The printer can not handle " + element.eClass().getName() + " elements", element);
	}
	
	protected org.emftext.language.refactoring.rolemapping.resource.rolemapping.mopp.RolemappingReferenceResolverSwitch getReferenceResolverSwitch() {
		return (org.emftext.language.refactoring.rolemapping.resource.rolemapping.mopp.RolemappingReferenceResolverSwitch) new org.emftext.language.refactoring.rolemapping.resource.rolemapping.mopp.RolemappingMetaInformation().getReferenceResolverSwitch();
	}
	
	protected void addWarningToResource(final String errorMessage, org.eclipse.emf.ecore.EObject cause) {
		org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTextResource resource = getResource();
		if (resource == null) {
			// the resource can be null if the printer is used stand alone
			return;
		}
		resource.addProblem(new org.emftext.language.refactoring.rolemapping.resource.rolemapping.mopp.RolemappingProblem(errorMessage, org.emftext.language.refactoring.rolemapping.resource.rolemapping.RolemappingEProblemType.PRINT_PROBLEM, org.emftext.language.refactoring.rolemapping.resource.rolemapping.RolemappingEProblemSeverity.WARNING), cause);
	}
	
	public void setOptions(java.util.Map<?,?> options) {
		this.options = options;
	}
	
	public java.util.Map<?,?> getOptions() {
		return options;
	}
	
	public void setEncoding(String encoding) {
		if (encoding != null) {
			this.encoding = encoding;
		}
	}
	
	public String getEncoding() {
		return encoding;
	}
	
	public org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTextResource getResource() {
		return resource;
	}
	
	/**
	 * Calls {@link #doPrint(EObject, PrintWriter, String)} and writes the result to
	 * the underlying output stream.
	 */
	public void print(org.eclipse.emf.ecore.EObject element) throws java.io.IOException {
		java.io.PrintWriter out = new java.io.PrintWriter(new java.io.OutputStreamWriter(new java.io.BufferedOutputStream(outputStream), encoding));
		doPrint(element, out, "");
		out.flush();
		out.close();
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_RoleMappingModel(org.emftext.language.refactoring.rolemapping.RoleMappingModel element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(3);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__TARGET_METAMODEL));
		printCountingMap.put("targetMetamodel", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__MAPPINGS));
		printCountingMap.put("mappings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__IMPORTS));
		printCountingMap.put("imports", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("ROLEMODELMAPPING");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("FOR");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("targetMetamodel");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__TARGET_METAMODEL));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_60_62");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRoleMappingModelTargetMetamodelReferenceResolver().deResolve((org.eclipse.emf.ecore.EPackage) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__TARGET_METAMODEL)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__TARGET_METAMODEL), element));
				out.print(" ");
			}
			printCountingMap.put("targetMetamodel", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_emftext_language_refactoring_rolemapping_RoleMappingModel_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("mappings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__MAPPINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("mappings", 0);
		}
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_RoleMappingModel_0(org.emftext.language.refactoring.rolemapping.RoleMappingModel element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("IMPORTS");
		out.print(" ");
		// DEFINITION PART BEGINS (CompoundDefinition)
		print_org_emftext_language_refactoring_rolemapping_RoleMappingModel_0_0(element, localtab, out, printCountingMap);
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_RoleMappingModel_0_0(org.emftext.language.refactoring.rolemapping.RoleMappingModel element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("imports");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__IMPORTS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_60_62");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRoleMappingModelImportsReferenceResolver().deResolve((org.eclipse.emf.ecore.EPackage) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__IMPORTS)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING_MODEL__IMPORTS), element));
				out.print(" ");
			}
			printCountingMap.put("imports", 0);
		}
	}
	
	
	public void print_org_emftext_language_refactoring_rolemapping_RoleMapping(org.emftext.language.refactoring.rolemapping.RoleMapping element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__NAME));
		printCountingMap.put("name", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__MAPPED_ROLE_MODEL));
		printCountingMap.put("mappedRoleModel", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__ROLE_TO_METAELEMENT));
		printCountingMap.put("roleToMetaelement", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__OWNING_MAPPING_MODEL));
		printCountingMap.put("owningMappingModel", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (CompoundDefinition)
		print_org_emftext_language_refactoring_rolemapping_RoleMapping_0(element, localtab, out, printCountingMap);
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("maps");
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderInQuotes)
		count = printCountingMap.get("mappedRoleModel");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__MAPPED_ROLE_MODEL));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_60_62");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getRoleMappingMappedRoleModelReferenceResolver().deResolve((org.emftext.language.refactoring.roles.RoleModel) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__MAPPED_ROLE_MODEL)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__MAPPED_ROLE_MODEL), element));
			}
			printCountingMap.put("mappedRoleModel", count - 1);
		}
		// DEFINITION PART BEGINS (WhiteSpaces)
		out.print(" ");
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		localtab += "	";
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("roleToMetaelement");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__ROLE_TO_METAELEMENT));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("roleToMetaelement", 0);
		}
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_RoleMapping_0(org.emftext.language.refactoring.rolemapping.RoleMapping element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		int alt = -1;
		alt = 0;
		int matches = 		matchCount(printCountingMap, java.util.Arrays.asList(		"name"		));
		int tempMatchCount;
		tempMatchCount = 		matchCount(printCountingMap, java.util.Arrays.asList(		"name"		));
		if (tempMatchCount > matches) {
			alt = 1;
			matches = tempMatchCount;
		}
		switch(alt) {
			case 1:			{
				// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
				count = printCountingMap.get("name");
				if (count > 0) {
					Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__NAME));
					if (o != null) {
						org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
						resolver.setOptions(getOptions());
						out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__NAME), element));
						out.print(" ");
					}
					printCountingMap.put("name", count - 1);
				}
			}
			break;
			default:			// DEFINITION PART BEGINS (PlaceholderInQuotes)
			count = printCountingMap.get("name");
			if (count > 0) {
				Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__NAME));
				if (o != null) {
					org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("QUOTED_34_34");
					resolver.setOptions(getOptions());
					out.print(resolver.deResolve((Object) o, element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ROLE_MAPPING__NAME), element));
					out.print(" ");
				}
				printCountingMap.put("name", count - 1);
			}
		}
	}
	
	
	public void print_org_emftext_language_refactoring_rolemapping_ConcreteMapping(org.emftext.language.refactoring.rolemapping.ConcreteMapping element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(4);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ROLE));
		printCountingMap.put("role", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__METACLASS));
		printCountingMap.put("metaclass", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__COLLABORATION_MAPPINGS));
		printCountingMap.put("collaborationMappings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ATTRIBUTE_MAPPINGS));
		printCountingMap.put("attributeMappings", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("role");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ROLE));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getConcreteMappingRoleReferenceResolver().deResolve((org.emftext.language.refactoring.roles.Role) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ROLE)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ROLE), element));
				out.print(" ");
			}
			printCountingMap.put("role", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(":=");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("metaclass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__METACLASS));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getConcreteMappingMetaclassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__METACLASS)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__METACLASS), element));
				out.print(" ");
			}
			printCountingMap.put("metaclass", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_emftext_language_refactoring_rolemapping_ConcreteMapping_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_emftext_language_refactoring_rolemapping_ConcreteMapping_1(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(";");
		out.print(" ");
		// DEFINITION PART BEGINS (LineBreak)
		out.println();
		out.print(localtab);
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_ConcreteMapping_0(org.emftext.language.refactoring.rolemapping.ConcreteMapping element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (CsString)
		out.print("(");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("attributeMappings");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ATTRIBUTE_MAPPINGS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("attributeMappings", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_emftext_language_refactoring_rolemapping_ConcreteMapping_0_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(")");
		out.print(" ");
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_ConcreteMapping_0_0(org.emftext.language.refactoring.rolemapping.ConcreteMapping element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(",");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("attributeMappings");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__ATTRIBUTE_MAPPINGS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("attributeMappings", count - 1);
		}
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_ConcreteMapping_1(org.emftext.language.refactoring.rolemapping.ConcreteMapping element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("{");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("collaborationMappings");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__COLLABORATION_MAPPINGS));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("collaborationMappings", count - 1);
		}
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("collaborationMappings");
		if (count > 0) {
			java.util.List<?> list = (java.util.List<?>)element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.CONCRETE_MAPPING__COLLABORATION_MAPPINGS));
			int index  = list.size() - count;
			if (index < 0) {
				index = 0;
			}
			java.util.ListIterator<?> it  = list.listIterator(index);
			while (it.hasNext()) {
				Object o = it.next();
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("collaborationMappings", 0);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("}");
		out.print(" ");
	}
	
	
	public void print_org_emftext_language_refactoring_rolemapping_CollaborationMapping(org.emftext.language.refactoring.rolemapping.CollaborationMapping element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(2);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__COLLABORATION));
		printCountingMap.put("collaboration", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__REFERENCE_META_CLASS_PAIR));
		printCountingMap.put("referenceMetaClassPair", temp == null ? 0 : ((java.util.Collection<?>) temp).size());
		// print collected hidden tokens
		int count;
		boolean iterate = true;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("collaboration");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__COLLABORATION));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getCollaborationMappingCollaborationReferenceResolver().deResolve((org.emftext.language.refactoring.roles.MultiplicityCollaboration) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__COLLABORATION)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__COLLABORATION), element));
				out.print(" ");
			}
			printCountingMap.put("collaboration", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(":=");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("referenceMetaClassPair");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__REFERENCE_META_CLASS_PAIR));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("referenceMetaClassPair", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		iterate = true;
		while (iterate) {
			sWriter = new java.io.StringWriter();
			out1 = new java.io.PrintWriter(sWriter);
			printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
			print_org_emftext_language_refactoring_rolemapping_CollaborationMapping_0(element, localtab, out1, printCountingMap1);
			if (printCountingMap.equals(printCountingMap1)) {
				iterate = false;
				out1.close();
			} else {
				out1.flush();
				out1.close();
				out.print(sWriter.toString());
				printCountingMap.putAll(printCountingMap1);
			}
		}
		// DEFINITION PART BEGINS (CsString)
		out.print(";");
		out.print(" ");
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_CollaborationMapping_0(org.emftext.language.refactoring.rolemapping.CollaborationMapping element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		String localtab = outertab;
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print("->");
		out.print(" ");
		// DEFINITION PART BEGINS (Containment)
		count = printCountingMap.get("referenceMetaClassPair");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.COLLABORATION_MAPPING__REFERENCE_META_CLASS_PAIR));
			java.util.List<?> list = (java.util.List<?>) o;
			int index = list.size() - count;
			if (index >= 0) {
				o = list.get(index);
			} else {
				o = null;
			}
			if (o != null) {
				doPrint((org.eclipse.emf.ecore.EObject) o, out, localtab);
			}
			printCountingMap.put("referenceMetaClassPair", count - 1);
		}
	}
	
	
	public void print_org_emftext_language_refactoring_rolemapping_ReferenceMetaClassPair(org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair element, String outertab, java.io.PrintWriter out) {
		String localtab = outertab;
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(2);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__REFERENCE));
		printCountingMap.put("reference", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__META_CLASS));
		printCountingMap.put("metaClass", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		java.io.StringWriter sWriter = null;
		java.io.PrintWriter out1 = null;
		java.util.Map<String, Integer> printCountingMap1 = null;
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("reference");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__REFERENCE));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReferenceMetaClassPairReferenceReferenceResolver().deResolve((org.eclipse.emf.ecore.EReference) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__REFERENCE)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__REFERENCE), element));
				out.print(" ");
			}
			printCountingMap.put("reference", count - 1);
		}
		// DEFINITION PART BEGINS (CompoundDefinition)
		sWriter = new java.io.StringWriter();
		out1 = new java.io.PrintWriter(sWriter);
		printCountingMap1 = new java.util.LinkedHashMap<String, Integer>(printCountingMap);
		print_org_emftext_language_refactoring_rolemapping_ReferenceMetaClassPair_0(element, localtab, out1, printCountingMap1);
		if (printCountingMap.equals(printCountingMap1)) {
			out1.close();
		} else {
			out1.flush();
			out1.close();
			out.print(sWriter.toString());
			printCountingMap.putAll(printCountingMap1);
		}
	}
	
	public void print_org_emftext_language_refactoring_rolemapping_ReferenceMetaClassPair_0(org.emftext.language.refactoring.rolemapping.ReferenceMetaClassPair element, String outertab, java.io.PrintWriter out, java.util.Map<String, Integer> printCountingMap) {
		int count;
		// DEFINITION PART BEGINS (CsString)
		out.print(":");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("metaClass");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__META_CLASS));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getReferenceMetaClassPairMetaClassReferenceResolver().deResolve((org.eclipse.emf.ecore.EClass) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__META_CLASS)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.REFERENCE_META_CLASS_PAIR__META_CLASS), element));
				out.print(" ");
			}
			printCountingMap.put("metaClass", count - 1);
		}
	}
	
	
	public void print_org_emftext_language_refactoring_rolemapping_AttributeMapping(org.emftext.language.refactoring.rolemapping.AttributeMapping element, String outertab, java.io.PrintWriter out) {
		// The printCountingMap contains a mapping from feature names to the number of
		// remaining elements that still need to be printed. The map is initialized with
		// the number of elements stored in each structural feature. For lists this is the
		// list size. For non-multiple features it is either 1 (if the feature is set) or
		// 0 (if the feature is null).
		java.util.Map<String, Integer> printCountingMap = new java.util.LinkedHashMap<String, Integer>(2);
		Object temp;
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__ROLE_ATTRIBUTE));
		printCountingMap.put("roleAttribute", temp == null ? 0 : 1);
		temp = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__CLASS_ATTRIBUTE));
		printCountingMap.put("classAttribute", temp == null ? 0 : 1);
		// print collected hidden tokens
		int count;
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("roleAttribute");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__ROLE_ATTRIBUTE));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getAttributeMappingRoleAttributeReferenceResolver().deResolve((org.emftext.language.refactoring.roles.RoleAttribute) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__ROLE_ATTRIBUTE)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__ROLE_ATTRIBUTE), element));
				out.print(" ");
			}
			printCountingMap.put("roleAttribute", count - 1);
		}
		// DEFINITION PART BEGINS (CsString)
		out.print("->");
		out.print(" ");
		// DEFINITION PART BEGINS (PlaceholderUsingSpecifiedToken)
		count = printCountingMap.get("classAttribute");
		if (count > 0) {
			Object o = element.eGet(element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__CLASS_ATTRIBUTE));
			if (o != null) {
				org.emftext.language.refactoring.rolemapping.resource.rolemapping.IRolemappingTokenResolver resolver = tokenResolverFactory.createTokenResolver("IDENT");
				resolver.setOptions(getOptions());
				out.print(resolver.deResolve(getReferenceResolverSwitch() == null ? null : getReferenceResolverSwitch().getAttributeMappingClassAttributeReferenceResolver().deResolve((org.eclipse.emf.ecore.EAttribute) o, element, (org.eclipse.emf.ecore.EReference) element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__CLASS_ATTRIBUTE)), element.eClass().getEStructuralFeature(org.emftext.language.refactoring.rolemapping.RolemappingPackage.ATTRIBUTE_MAPPING__CLASS_ATTRIBUTE), element));
				out.print(" ");
			}
			printCountingMap.put("classAttribute", count - 1);
		}
	}
	
	
}
