/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.refactoring.roles.resource.rolestext.mopp;

public class RolestextReferenceResolverSwitch implements org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolverSwitch {
	
	/**
	 * This map stores a copy of the options the were set for loading the resource.
	 */
	private java.util.Map<Object, Object> options;
	
	protected org.emftext.language.refactoring.roles.resource.rolestext.analysis.CollaborationSourceReferenceResolver collaborationSourceReferenceResolver = new org.emftext.language.refactoring.roles.resource.rolestext.analysis.CollaborationSourceReferenceResolver();
	protected org.emftext.language.refactoring.roles.resource.rolestext.analysis.CollaborationTargetReferenceResolver collaborationTargetReferenceResolver = new org.emftext.language.refactoring.roles.resource.rolestext.analysis.CollaborationTargetReferenceResolver();
	
	public org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver<org.emftext.language.refactoring.roles.Collaboration, org.emftext.language.refactoring.roles.Role> getCollaborationSourceReferenceResolver() {
		return getResolverChain(org.emftext.language.refactoring.roles.RolesPackage.eINSTANCE.getCollaboration_Source(), collaborationSourceReferenceResolver);
	}
	
	public org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver<org.emftext.language.refactoring.roles.Collaboration, org.emftext.language.refactoring.roles.Role> getCollaborationTargetReferenceResolver() {
		return getResolverChain(org.emftext.language.refactoring.roles.RolesPackage.eINSTANCE.getCollaboration_Target(), collaborationTargetReferenceResolver);
	}
	
	public void setOptions(java.util.Map<?, ?> options) {
		if (options != null) {
			this.options = new java.util.LinkedHashMap<Object, Object>();
			this.options.putAll(options);
		}
		collaborationSourceReferenceResolver.setOptions(options);
		collaborationTargetReferenceResolver.setOptions(options);
	}
	
	public void resolveFuzzy(String identifier, org.eclipse.emf.ecore.EObject container, org.eclipse.emf.ecore.EReference reference, int position, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolveResult<org.eclipse.emf.ecore.EObject> result) {
		if (container == null) {
			return;
		}
		if (org.emftext.language.refactoring.roles.RolesPackage.eINSTANCE.getCollaboration().isInstance(container)) {
			RolestextFuzzyResolveResult<org.emftext.language.refactoring.roles.Role> frr = new RolestextFuzzyResolveResult<org.emftext.language.refactoring.roles.Role>(result);
			String referenceName = reference.getName();
			org.eclipse.emf.ecore.EStructuralFeature feature = container.eClass().getEStructuralFeature(referenceName);
			if (feature != null && feature instanceof org.eclipse.emf.ecore.EReference && referenceName != null && referenceName.equals("source")) {
				collaborationSourceReferenceResolver.resolve(identifier, (org.emftext.language.refactoring.roles.Collaboration) container, (org.eclipse.emf.ecore.EReference) feature, position, true, frr);
			}
		}
		if (org.emftext.language.refactoring.roles.RolesPackage.eINSTANCE.getCollaboration().isInstance(container)) {
			RolestextFuzzyResolveResult<org.emftext.language.refactoring.roles.Role> frr = new RolestextFuzzyResolveResult<org.emftext.language.refactoring.roles.Role>(result);
			String referenceName = reference.getName();
			org.eclipse.emf.ecore.EStructuralFeature feature = container.eClass().getEStructuralFeature(referenceName);
			if (feature != null && feature instanceof org.eclipse.emf.ecore.EReference && referenceName != null && referenceName.equals("target")) {
				collaborationTargetReferenceResolver.resolve(identifier, (org.emftext.language.refactoring.roles.Collaboration) container, (org.eclipse.emf.ecore.EReference) feature, position, true, frr);
			}
		}
	}
	
	public org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver<? extends org.eclipse.emf.ecore.EObject, ? extends org.eclipse.emf.ecore.EObject> getResolver(org.eclipse.emf.ecore.EStructuralFeature reference) {
		if (reference == org.emftext.language.refactoring.roles.RolesPackage.eINSTANCE.getCollaboration_Source()) {
			return getResolverChain(reference, collaborationSourceReferenceResolver);
		}
		if (reference == org.emftext.language.refactoring.roles.RolesPackage.eINSTANCE.getCollaboration_Target()) {
			return getResolverChain(reference, collaborationTargetReferenceResolver);
		}
		return null;
	}
	
	@SuppressWarnings({"rawtypes", "unchecked"})	
	public <ContainerType extends org.eclipse.emf.ecore.EObject, ReferenceType extends org.eclipse.emf.ecore.EObject> org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver<ContainerType, ReferenceType> getResolverChain(org.eclipse.emf.ecore.EStructuralFeature reference, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver<ContainerType, ReferenceType> originalResolver) {
		if (options == null) {
			return originalResolver;
		}
		Object value = options.get(org.emftext.language.refactoring.roles.resource.rolestext.IRolestextOptions.ADDITIONAL_REFERENCE_RESOLVERS);
		if (value == null) {
			return originalResolver;
		}
		if (!(value instanceof java.util.Map)) {
			// send this to the error log
			new org.emftext.language.refactoring.roles.resource.rolestext.util.RolestextRuntimeUtil().logWarning("Found value with invalid type for option " + org.emftext.language.refactoring.roles.resource.rolestext.IRolestextOptions.ADDITIONAL_REFERENCE_RESOLVERS + " (expected " + java.util.Map.class.getName() + ", but was " + value.getClass().getName() + ")", null);
			return originalResolver;
		}
		java.util.Map<?,?> resolverMap = (java.util.Map<?,?>) value;
		Object resolverValue = resolverMap.get(reference);
		if (resolverValue == null) {
			return originalResolver;
		}
		if (resolverValue instanceof org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver) {
			org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver replacingResolver = (org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver) resolverValue;
			if (replacingResolver instanceof org.emftext.language.refactoring.roles.resource.rolestext.IRolestextDelegatingReferenceResolver) {
				// pass original resolver to the replacing one
				((org.emftext.language.refactoring.roles.resource.rolestext.IRolestextDelegatingReferenceResolver) replacingResolver).setDelegate(originalResolver);
			}
			return replacingResolver;
		} else if (resolverValue instanceof java.util.Collection) {
			java.util.Collection replacingResolvers = (java.util.Collection) resolverValue;
			org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver replacingResolver = originalResolver;
			for (Object next : replacingResolvers) {
				if (next instanceof org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceCache) {
					org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver nextResolver = (org.emftext.language.refactoring.roles.resource.rolestext.IRolestextReferenceResolver) next;
					if (nextResolver instanceof org.emftext.language.refactoring.roles.resource.rolestext.IRolestextDelegatingReferenceResolver) {
						// pass original resolver to the replacing one
						((org.emftext.language.refactoring.roles.resource.rolestext.IRolestextDelegatingReferenceResolver) nextResolver).setDelegate(replacingResolver);
					}
					replacingResolver = nextResolver;
				} else {
					// The collection contains a non-resolver. Send a warning to the error log.
					new org.emftext.language.refactoring.roles.resource.rolestext.util.RolestextRuntimeUtil().logWarning("Found value with invalid type in value map for option " + org.emftext.language.refactoring.roles.resource.rolestext.IRolestextOptions.ADDITIONAL_REFERENCE_RESOLVERS + " (expected " + org.emftext.language.refactoring.roles.resource.rolestext.IRolestextDelegatingReferenceResolver.class.getName() + ", but was " + next.getClass().getName() + ")", null);
				}
			}
			return replacingResolver;
		} else {
			// The value for the option ADDITIONAL_REFERENCE_RESOLVERS has an unknown type.
			new org.emftext.language.refactoring.roles.resource.rolestext.util.RolestextRuntimeUtil().logWarning("Found value with invalid type in value map for option " + org.emftext.language.refactoring.roles.resource.rolestext.IRolestextOptions.ADDITIONAL_REFERENCE_RESOLVERS + " (expected " + org.emftext.language.refactoring.roles.resource.rolestext.IRolestextDelegatingReferenceResolver.class.getName() + ", but was " + resolverValue.getClass().getName() + ")", null);
			return originalResolver;
		}
	}
	
}
