/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.refactoring.roles.resource.rolestext.mopp;

/**
 * The RolestextTokenResolverFactory class provides access to all generated token
 * resolvers. By giving the name of a defined token, the corresponding resolve can
 * be obtained. Despite the fact that this class is called TokenResolverFactory is
 * does NOT create new token resolvers whenever a client calls methods to obtain a
 * resolver. Rather, this class maintains a map of all resolvers and creates each
 * resolver at most once.
 */
public class RolestextTokenResolverFactory implements org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolverFactory {
	
	private java.util.Map<String, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver> tokenName2TokenResolver;
	private java.util.Map<String, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver> featureName2CollectInTokenResolver;
	private static org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver defaultResolver = new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextDefaultTokenResolver();
	
	public RolestextTokenResolverFactory() {
		tokenName2TokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver>();
		featureName2CollectInTokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver>();
		registerTokenResolver("ML_COMMENT", new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextML_COMMENTTokenResolver());
		registerTokenResolver("ROLE_MODIFIER", new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextROLE_MODIFIERTokenResolver());
		registerTokenResolver("RELATION_MODIFIER", new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextRELATION_MODIFIERTokenResolver());
		registerTokenResolver("UPPER_IDENTIFIER", new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextUPPER_IDENTIFIERTokenResolver());
		registerTokenResolver("LOWER_IDENTIFIER", new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextLOWER_IDENTIFIERTokenResolver());
		registerTokenResolver("NUMBER", new org.emftext.language.refactoring.roles.resource.rolestext.analysis.RolestextNUMBERTokenResolver());
	}
	
	public org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver createTokenResolver(String tokenName) {
		return internalCreateResolver(tokenName2TokenResolver, tokenName);
	}
	
	public org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver createCollectInTokenResolver(String featureName) {
		return internalCreateResolver(featureName2CollectInTokenResolver, featureName);
	}
	
	protected boolean registerTokenResolver(String tokenName, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver resolver){
		return internalRegisterTokenResolver(tokenName2TokenResolver, tokenName, resolver);
	}
	
	protected boolean registerCollectInTokenResolver(String featureName, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver resolver){
		return internalRegisterTokenResolver(featureName2CollectInTokenResolver, featureName, resolver);
	}
	
	protected org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver deRegisterTokenResolver(String tokenName){
		return tokenName2TokenResolver.remove(tokenName);
	}
	
	private org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver internalCreateResolver(java.util.Map<String, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver> resolverMap, String key) {
		if (resolverMap.containsKey(key)){
			return resolverMap.get(key);
		} else {
			return defaultResolver;
		}
	}
	
	private boolean internalRegisterTokenResolver(java.util.Map<String, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver> resolverMap, String key, org.emftext.language.refactoring.roles.resource.rolestext.IRolestextTokenResolver resolver) {
		if (!resolverMap.containsKey(key)) {
			resolverMap.put(key,resolver);
			return true;
		}
		return false;
	}
	
}
