/**
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 *  
 */
package org.emftext.language.refactoring.roles.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import org.eclipse.emf.ecore.util.Switch;

import org.emftext.language.refactoring.roles.*;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see org.emftext.language.refactoring.roles.RolesPackage
 * @generated
 */
public class RolesSwitch<T> extends Switch<T>
{
  /**
   * The cached model package
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  protected static RolesPackage modelPackage;

  /**
   * Creates an instance of the switch.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @generated
   */
  public RolesSwitch()
  {
    if (modelPackage == null)
    {
      modelPackage = RolesPackage.eINSTANCE;
    }
  }

  /**
   * Checks whether this is a switch for the given package.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @parameter ePackage the package in question.
   * @return whether this is a switch for the given package.
   * @generated
   */
  @Override
  protected boolean isSwitchFor(EPackage ePackage)
  {
    return ePackage == modelPackage;
  }

  /**
   * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
   * <!-- begin-user-doc -->
   * <!-- end-user-doc -->
   * @return the first non-null result returned by a <code>caseXXX</code> call.
   * @generated
   */
  @Override
  protected T doSwitch(int classifierID, EObject theEObject)
  {
    switch (classifierID)
    {
      case RolesPackage.NAMED_ELEMENT:
      {
        NamedElement namedElement = (NamedElement)theEObject;
        T result = caseNamedElement(namedElement);
        if (result == null) result = caseCommentable(namedElement);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.COLLABORATION:
      {
        Collaboration collaboration = (Collaboration)theEObject;
        T result = caseCollaboration(collaboration);
        if (result == null) result = caseRoleFeature(collaboration);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE:
      {
        Role role = (Role)theEObject;
        T result = caseRole(role);
        if (result == null) result = caseNamedElement(role);
        if (result == null) result = caseCommentable(role);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_MODEL:
      {
        RoleModel roleModel = (RoleModel)theEObject;
        T result = caseRoleModel(roleModel);
        if (result == null) result = caseNamedElement(roleModel);
        if (result == null) result = caseCommentable(roleModel);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.MULTIPLICITY:
      {
        Multiplicity multiplicity = (Multiplicity)theEObject;
        T result = caseMultiplicity(multiplicity);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.MULTIPLICITY_COLLABORATION:
      {
        MultiplicityCollaboration multiplicityCollaboration = (MultiplicityCollaboration)theEObject;
        T result = caseMultiplicityCollaboration(multiplicityCollaboration);
        if (result == null) result = caseCollaboration(multiplicityCollaboration);
        if (result == null) result = caseRoleFeature(multiplicityCollaboration);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_PROHIBITION:
      {
        RoleProhibition roleProhibition = (RoleProhibition)theEObject;
        T result = caseRoleProhibition(roleProhibition);
        if (result == null) result = caseCollaboration(roleProhibition);
        if (result == null) result = caseRoleFeature(roleProhibition);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_ASSOCIATION:
      {
        RoleAssociation roleAssociation = (RoleAssociation)theEObject;
        T result = caseRoleAssociation(roleAssociation);
        if (result == null) result = caseMultiplicityCollaboration(roleAssociation);
        if (result == null) result = caseCollaboration(roleAssociation);
        if (result == null) result = caseRoleFeature(roleAssociation);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_COMPOSITION:
      {
        RoleComposition roleComposition = (RoleComposition)theEObject;
        T result = caseRoleComposition(roleComposition);
        if (result == null) result = caseMultiplicityCollaboration(roleComposition);
        if (result == null) result = caseCollaboration(roleComposition);
        if (result == null) result = caseRoleFeature(roleComposition);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_IMPLICATION:
      {
        RoleImplication roleImplication = (RoleImplication)theEObject;
        T result = caseRoleImplication(roleImplication);
        if (result == null) result = caseCollaboration(roleImplication);
        if (result == null) result = caseRoleFeature(roleImplication);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_ATTRIBUTE:
      {
        RoleAttribute roleAttribute = (RoleAttribute)theEObject;
        T result = caseRoleAttribute(roleAttribute);
        if (result == null) result = caseNamedElement(roleAttribute);
        if (result == null) result = caseRoleFeature(roleAttribute);
        if (result == null) result = caseCommentable(roleAttribute);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.ROLE_FEATURE:
      {
        RoleFeature roleFeature = (RoleFeature)theEObject;
        T result = caseRoleFeature(roleFeature);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      case RolesPackage.COMMENTABLE:
      {
        Commentable commentable = (Commentable)theEObject;
        T result = caseCommentable(commentable);
        if (result == null) result = defaultCase(theEObject);
        return result;
      }
      default: return defaultCase(theEObject);
    }
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Named Element</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Named Element</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseNamedElement(NamedElement object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Collaboration</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Collaboration</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCollaboration(Collaboration object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRole(Role object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Model</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Model</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleModel(RoleModel object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Multiplicity</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Multiplicity</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseMultiplicity(Multiplicity object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Multiplicity Collaboration</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Multiplicity Collaboration</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseMultiplicityCollaboration(MultiplicityCollaboration object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Prohibition</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Prohibition</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleProhibition(RoleProhibition object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Association</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Association</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleAssociation(RoleAssociation object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Composition</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Composition</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleComposition(RoleComposition object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Implication</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Implication</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleImplication(RoleImplication object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Attribute</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Attribute</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleAttribute(RoleAttribute object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Role Feature</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Role Feature</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseRoleFeature(RoleFeature object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>Commentable</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>Commentable</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
   * @generated
   */
  public T caseCommentable(Commentable object)
  {
    return null;
  }

  /**
   * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
   * <!-- begin-user-doc -->
   * This implementation returns null;
   * returning a non-null result will terminate the switch, but this is the last case anyway.
   * <!-- end-user-doc -->
   * @param object the target of the switch.
   * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
   * @see #doSwitch(org.eclipse.emf.ecore.EObject)
   * @generated
   */
  @Override
  public T defaultCase(EObject object)
  {
    return null;
  }

} //RolesSwitch
