/*******************************************************************************
 * Copyright (c) 2006-2012
 * Software Technology Group, Dresden University of Technology
 * DevBoost GmbH, Berlin, Amtsgericht Charlottenburg, HRB 140026
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Software Technology Group - TU Dresden, Germany;
 *   DevBoost GmbH - Berlin, Germany
 *      - initial API and implementation
 ******************************************************************************/

package org.emftext.language.refactoring.specification.resource.mopp;

/**
 * The RefspecTokenResolverFactory class provides access to all generated token
 * resolvers. By giving the name of a defined token, the corresponding resolve can
 * be obtained. Despite the fact that this class is called TokenResolverFactory is
 * does NOT create new token resolvers whenever a client calls methods to obtain a
 * resolver. Rather, this class maintains a map of all resolvers and creates each
 * resolver at most once.
 */
public class RefspecTokenResolverFactory implements org.emftext.language.refactoring.specification.resource.IRefspecTokenResolverFactory {
	
	private java.util.Map<String, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver> tokenName2TokenResolver;
	private java.util.Map<String, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver> featureName2CollectInTokenResolver;
	private static org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver defaultResolver = new org.emftext.language.refactoring.specification.resource.analysis.RefspecDefaultTokenResolver();
	
	public RefspecTokenResolverFactory() {
		tokenName2TokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver>();
		featureName2CollectInTokenResolver = new java.util.LinkedHashMap<String, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver>();
		registerTokenResolver("INTEGER", new org.emftext.language.refactoring.specification.resource.analysis.RefspecINTEGERTokenResolver());
		registerTokenResolver("CONSTANTS", new org.emftext.language.refactoring.specification.resource.analysis.RefspecCONSTANTSTokenResolver());
		registerTokenResolver("UPPER_IDENTIFIER", new org.emftext.language.refactoring.specification.resource.analysis.RefspecUPPER_IDENTIFIERTokenResolver());
		registerTokenResolver("LOWER_IDENTIFIER", new org.emftext.language.refactoring.specification.resource.analysis.RefspecLOWER_IDENTIFIERTokenResolver());
		registerTokenResolver("DOT_NOTATION", new org.emftext.language.refactoring.specification.resource.analysis.RefspecDOT_NOTATIONTokenResolver());
		registerTokenResolver("QUOTED_60_62", new org.emftext.language.refactoring.specification.resource.analysis.RefspecQUOTED_60_62TokenResolver());
	}
	
	public org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver createTokenResolver(String tokenName) {
		return internalCreateResolver(tokenName2TokenResolver, tokenName);
	}
	
	public org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver createCollectInTokenResolver(String featureName) {
		return internalCreateResolver(featureName2CollectInTokenResolver, featureName);
	}
	
	protected boolean registerTokenResolver(String tokenName, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver resolver){
		return internalRegisterTokenResolver(tokenName2TokenResolver, tokenName, resolver);
	}
	
	protected boolean registerCollectInTokenResolver(String featureName, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver resolver){
		return internalRegisterTokenResolver(featureName2CollectInTokenResolver, featureName, resolver);
	}
	
	protected org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver deRegisterTokenResolver(String tokenName){
		return tokenName2TokenResolver.remove(tokenName);
	}
	
	private org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver internalCreateResolver(java.util.Map<String, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver> resolverMap, String key) {
		if (resolverMap.containsKey(key)){
			return resolverMap.get(key);
		} else {
			return defaultResolver;
		}
	}
	
	private boolean internalRegisterTokenResolver(java.util.Map<String, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver> resolverMap, String key, org.emftext.language.refactoring.specification.resource.IRefspecTokenResolver resolver) {
		if (!resolverMap.containsKey(key)) {
			resolverMap.put(key,resolver);
			return true;
		}
		return false;
	}
	
}
